/* ginny mak - linux version of signal strength
 * driver, based on iwconfig and iwspy wireless extension source
 */

#include <ctype.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "cSigStat.h"
#include "linux.h"

#define FAILURE -1
#define SUCCESS 0

int lucentdbm = 0;

static int
get_hermes_signal(char *dev,  int *signal,  int *noise,  int *quality);

/**************************************************
 * Implementation of class cSigStat               *
 **************************************************/
cSigStat::cSigStat() 
{
  if( (mSkfd = sockets_open()) < 0 )
    exit(FAILURE);
  mIfname = NULL;
}

cSigStat::~cSigStat() 
{}


int cSigStat::Initialize( char *ifname, int flags)
{
  struct iwreq wrq;

  if (ifname == NULL)
	return FAILURE; 
  mIfname = new char[strlen(ifname)+1];
  
  strcpy( mIfname, ifname );

  // detect network card here
  strcpy( wrq.ifr_name, mIfname );
  if( ioctl(mSkfd, SIOCGIWNAME, &wrq) < 0 ) {
	return FAILURE;
  }

  mInitialized = true;
  return SUCCESS;
}


/*
 * returns:
 *	MANAGED_MODE or ADHOC_MODE or
 *	error (<0)
 */
int cSigStat::wifi_getmode() 
{
   struct iwreq wrq;

   if(!mInitialized)
	 return FAILURE;
   strcpy( wrq.ifr_name, mIfname );

   if( ioctl( mSkfd, SIOCGIWMODE, &wrq ) >= 0 )  {
     if (wrq.u.mode == 2) {
       mAdhoc = false;
       return(MANAGED_MODE);
     }
     if (wrq.u.mode == 1)
       return(ADHOC_MODE);
   }
   return FAILURE;
}



char *cSigStat::wifi_getessid() 
{
  static char essid[IW_ESSID_MAX_SIZE+1];
  struct iwreq wrq;

  if( !mInitialized )
	return NULL;

  // char *essid = (char *)malloc(sizeof(char)*(IW_ESSID_MAX_SIZE+1));
  strcpy( wrq.ifr_name, mIfname );
  wrq.u.essid.pointer = (caddr_t)essid;
  wrq.u.essid.length = 0;
  wrq.u.essid.flags = 0;
  if( ioctl(mSkfd, SIOCGIWESSID, &wrq) >= 0 )
	return essid;
  else
	return NULL;
  
}

char * cSigStat::wifi_getchannel()
{
  float freq;

  struct iwreq wrq;
  static char channel[15];

  if( !mInitialized )
	return NULL;
  //char *channel = new char(15);
  strcpy( channel, "");
  strcpy( wrq.ifr_name, mIfname );
  if( ioctl( mSkfd, SIOCGIWFREQ, &wrq ) >= 0 ) {
	freq = freq2float(&(wrq.u.freq));
	if( freq < KILO )
	  sprintf( channel, "%d", (int)freq );
	else {
	  if( freq >= GIGA ) {
		freq = freq/MEGA;
		freq2channel( freq, channel );
	  }
	}
	return channel;
  }
  else {
	//delete channel;
	return NULL;
  }

}


unsigned char *cSigStat::wifi_getmac( unsigned char *addr ) 
{
  struct iwreq wrq;
  
  strcpy( wrq.ifr_name, mIfname );
  if( ioctl( mSkfd, SIOCGIWAP, &wrq ) >= 0 ) {
	memcpy( addr, &(wrq.u.ap_addr.sa_data), sizeof(wrq.u.ap_addr.sa_data));
	return addr;
  }
  else 
	return NULL;
}


/*
 * pad buffers installed around iw_range range variable
 * as a workaround for apparent kernel miscue.  ioctl
 * structure reads out something that is apparently BIGGER
 * than iw_range.  This is definitely true on the arm/ipaq,
 * and may be true on x86 as well.  Nov. 12. 2001, jrb.
 */
int cSigStat::wifi_getrange( int *maxLevel, int *maxNoise, int *maxQual,
	int *minLevel, int *minNoise, int *minQual )
{
  char pad1[1024];
  struct iw_range range;
  char pad2[1024];
  if( get_range_info( mSkfd, mIfname,&range ) == 0 ) {
	
	/* unsigned chars and negative arithmetic is a very
	 * bad combination
	 */
	if (range.max_qual.level == 153)	/* lucent/dbm */
	{
		lucentdbm = 1;
		*maxLevel = 138;  /* sans dbm which is -149 */
		*maxNoise = 138;  /* sans dbm which is -149 */
		*maxQual = range.max_qual.qual;
		*minLevel = 47;
		*minNoise = 47;
		*minQual = 0;
		return SUCCESS;
	}
	*maxLevel = range.max_qual.level;
	*maxNoise = range.max_qual.noise;
	*maxQual = range.max_qual.qual;
	/* wild assumption follows
	*/
	*minLevel = *minNoise = *minQual = 0;
	return SUCCESS;
  }
  else {
	*maxLevel = 0;
	*maxNoise = 0;
	*maxQual = 0;
	return FAILURE;
  }
}

int cSigStat::wifi_getsignal( int *signal, int *noise, int *qual ) 
{
  iwstats stats;
  int rc;

  iw_getstats(mIfname, &stats);
  *qual = stats.qual.qual;
  if (lucentdbm) {
	/* try for orinoco info 1st
	*/
	rc = get_hermes_signal(mIfname,  signal,  noise,  qual);
	if (rc == SUCCESS) {
		*qual = *signal - *noise;       /* don't leave up to driver/s */
		return(SUCCESS);
	}
	
	/* 107 due to magic unsigned char/dbm madness
	*/
	*signal = stats.qual.level - 107;
	*noise = stats.qual.noise - 107;
	*qual = *signal - *noise;       /* don't leave up to driver/s */
  	return SUCCESS;
  }
  return SUCCESS;
}

int cSigStat::wifi_gettable( int count, sigtuple *sigtupleAr )
{
  struct iwreq wrq;
  int MAX_MAC = 20;
  char *macAddr[MAX_MAC];
  char *ipAddr[MAX_MAC];

  for( int i = 0; i < MAX_MAC; i++ ) {
    macAddr[i] = (char *)malloc( sizeof( char ) * 64 );
    ipAddr[i] = (char *)malloc( sizeof( char ) * 20 );
  }
  
  char buffer[(sizeof(struct iw_quality) +
			   sizeof(struct sockaddr))*IW_MAX_SPY];
  struct sockaddr hwa[IW_MAX_SPY];
  struct iw_quality qual[IW_MAX_SPY];
  int n;
  int i;

  /* get the statistics */
  strncpy( wrq.ifr_name, mIfname, IFNAMSIZ );
  wrq.u.data.pointer = (caddr_t) buffer;
  wrq.u.data.length = 0;
  wrq.u.data.flags = 0;
  if( ioctl( mSkfd, SIOCGIWSPY, &wrq ) < 0 )
	return FAILURE;
  if( check_addr_type( mSkfd, mIfname ) < 0 )
	return FAILURE;

  /* number of addresses */
  n = wrq.u.data.length;

  if( n == 0 )
	return 0;

  if( count > n )
	count = n;

  /* get list of socket addresses and list of signal statistics */
  memcpy( hwa, buffer, count*sizeof(struct sockaddr) );
  memcpy( qual, buffer+n*sizeof(struct sockaddr), count*sizeof(struct iw_quality));
  for( i = 0; i < count; i++ ) {
	memcpy( (sigtupleAr+i)->mac, hwa[i].sa_data, 6 );
	if (lucentdbm) {
		(sigtupleAr+i)->signal = qual[i].level - 107;
		(sigtupleAr+i)->noise = qual[i].noise - 107;
	}
	else {
		(sigtupleAr+i)->signal = qual[i].level;
		(sigtupleAr+i)->noise = qual[i].noise;
	}
	/* don't leave up to driver/s */
	(sigtupleAr+i)->quality = (sigtupleAr+i)->signal -
                                (sigtupleAr+i)->noise;


  }

  if( !mAdhoc ) {
     for( int i = 0; i < count; i++ )
      strcpy( macAddr[i], pr_ether( (sigtupleAr+i)->mac ) );
    getIps( count, macAddr, ipAddr );
  
    for (int i = 0 ; i < count; i++ ) {
      (sigtupleAr+i)->ip = inet_addr( ipAddr[i] );
    }
    mAdhoc = true;
  }
  for( int i = 0; i < MAX_MAC; i++ ) {
	delete macAddr[i];
	delete ipAddr[i];
  }

  return count;
}


/*
 * BUG: make sure iflist is not bigger than max
 * BUG: do not return localhost in list.
 */
int cSigStat::wifi_getifs( int *count,  char **iflist, int max)
{
  char buff[1024];
  struct ifconf ifc;
  struct ifreq *ifr;
  
  ifc.ifc_len = sizeof(buff);
  ifc.ifc_buf = buff;
  if( ioctl(mSkfd, SIOCGIFCONF, &ifc) < 0 ) {
    return FAILURE;
  }
  ifr = ifc.ifc_req;
  *count = ifc.ifc_len/sizeof(struct ifreq);
  /*
  if( *count == 0 || *count == 1 )
    return 0;
  else {
    for( int i = 0; i < *count; i++ ) {
	iflist[i] = (char *)malloc( sizeof(char) * 10 );
	strcpy( iflist[i], "");
    }
  }
  */

  int n = 0;
  for( int i = ifc.ifc_len/sizeof(struct ifreq); --i >= 0; ifr++){

    strcpy(iflist[n++], ifr->ifr_name);
   }
  return *count;
}


void cSigStat::wifi_close()
{
  close( mSkfd );
  mSkfd = -1;
  mInitialized = false;
}


/*
 * routine to determine if orinoco_cs used, and
 * then get signal strength value.  This routine
 * should go away when IPAQ/orinoco_cs driver gains
 * fix for current bugs in /proc/net/wireless.
 * Here we get the signal strength info from /proc/hermes/eth?/recs.txt
 *
 * FAILURE, means no luck, assume not orinoco_cs driver.
 */
static int
get_hermes_signal(char *dev,  int *signal,  int *noise,  int *quality)
{
	FILE *fd;
	char filename[1024];
	char linebuf[1024];
	char *opword;
	char *q;
	char *s;
	char *n;

	bzero(filename, 1024);
	strcat(filename, "/proc/hermes/");
	strcat(filename, dev);
	sprintf(filename, "%s/%s/%s", "/proc/hermes", dev, "recs");

#ifdef HERMESDEBUG
	printf("%s\n", filename);
#endif
	/*
	strcpy(filename, "recs.txt");
	*/

	fd = fopen(filename, "r");
	if (fd == NULL) {
		return(FAILURE);
	}
#ifdef HERMESDEBUG
	printf("fopen ok\n");
#endif

  	while(fgets(linebuf,1024,fd)) {
 		opword = (char *)strtok(linebuf, " \t");
		if (strcmp(opword, "COMMSQUALITY")==0) {
			while( (opword = (char *)strtok(NULL, " \t=:-")) != NULL)
			{
#ifdef HERMESDEBUG
				printf("%s\n", opword);
#endif
				/* value = qual-sig-noise */
				if (strcmp(opword, "value")==0) {
					opword = (char *)strtok(NULL, " \t=:-");
					q = opword;
					opword = (char *)strtok(NULL, " \t=:-");
					s = opword;
					opword = (char *)strtok(NULL, " \t=:-");
					n = opword;
					sscanf(q, "%x", quality);
					sscanf(s, "%x", signal);
					sscanf(n, "%x", noise);
					fclose(fd);
					return(SUCCESS);
				}
			}
			break;
		}
	}
	fclose(fd);
	return(FAILURE);
}

