#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include "cSigStat.h"
#include "linux.h"

#define FAILURE -1
#define SUCCESS 0

#define MAXTUPLES	20

int
main(int	argc,
     char **	argv)
{
  char *ifname;
  unsigned char macAddr[6];
  int maxLevel, maxNoise, maxQual;
  int minLevel, minNoise, minQual;
  sigtuple sigtupleAr[MAXTUPLES];
  char *mac[MAXTUPLES];
  char *ip[MAXTUPLES];
  char opt;
  int mode;
  int level=0; int  noise=0; int qual=0;
  int count = 0; int i = 0;
  int flag = 0;

  ifname = (char *)"eth0";
  if (argc == 2)
	  ifname = argv[1];

  for (int i = 0; i < MAXTUPLES; i++) {
	mac[i] = new char(64);
	ip[i] = new char(20);
  }

  cSigStat *sigStat = new cSigStat();
  if( sigStat->Initialize(ifname, flag) == FAILURE ) {
	printf("Unable to open connection to the NET kernel.\n");
	exit(-1);
  }

  mode = sigStat->wifi_getmode();
  printf("Mode: %d : ", sigStat->wifi_getmode());
  if (mode == MANAGED_MODE)
	printf("managed\n");
  else
	printf("ad-hoc\n");

  printf("Essid : %s\n", sigStat->wifi_getessid());

  char * freq = sigStat->wifi_getchannel();

  printf("freq %s\n", freq);

  printf("MAC address is %s\n", pr_ether(sigStat->wifi_getmac(macAddr)));

  printf("get range:\n");
  if ( sigStat->wifi_getrange( &maxLevel, &maxNoise, &maxQual,
		&minLevel, &minNoise, &minQual ) == SUCCESS ) {
	printf("MAX level %d, noise %d, quality %d\n", 
		maxLevel, maxNoise, maxQual);
	printf("MIN level %d, noise %d, quality %d\n", 
		minLevel, minNoise, minQual);
  }
  else
	printf("Cannot get the range.\n");

  printf("get statistics:\n");
  if( sigStat->wifi_getsignal( &level, &noise, &qual ) == SUCCESS )
	printf("level %d, noise %d, quality %d\n", level, noise, qual);
  else
	printf("Cannot get signal statistics.\n");
  
  printf("Get table entries:\n");
  if( (count = sigStat->wifi_gettable( MAXTUPLES, sigtupleAr )) > 0 ) {
	for( i = 0; i < count; i++ ) {
	  strcpy(mac[i], pr_ether( sigtupleAr[i].mac ) );
	  printf("MAC : %s: ", pr_ether( sigtupleAr[i].mac ) );
	  printf("signal %d, noise %d, quality %d\n", 
			 sigtupleAr[i].signal,
			 sigtupleAr[i].noise, 
			 sigtupleAr[i].quality);
	}
  }

  getIps( count, mac, ip );

  for( int i = 0 ; i < count; i++ ) {
    printf( "mac: %s ", pr_ether( sigtupleAr[i].mac ) );
    printf( "ip: %s\n", ip[i] );
  }

  /* Close the socket.
  */
  sigStat->wifi_close();

  return 0;
}

