#ifndef COMMON_H
#define COMMON_H

/* Standard headers */
#include <sys/types.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <math.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>		/* gethostbyname, getnetbyname */
#include <sys/socket.h>
#include <linux/if_arp.h>

/* This is our header selection. Try to hide the mess and the misery :-(
 * Please choose only one of the define...
 */

/* Wireless extensions */
#include <linux/wireless.h>

#if WIRELESS_EXT < 8
#error "Wireless Extension v9 or newer required :-(\n\
Use Wireless Tools v19 or update your kernel headers"
#endif

/************************ CONSTANTS & MACROS ************************/

/* Some usefull constants */
#define KILO	1e3
#define MEGA	1e6
#define GIGA	1e9

/****************************** TYPES ******************************/

/* Shortcuts */
typedef struct iw_statistics	iwstats;
typedef struct iw_range		iwrange;
typedef struct iw_param		iwparam;
typedef struct iw_freq		iwfreq;
typedef struct iw_priv_args	iwprivargs;
typedef struct sockaddr		sockaddr;


/**************************** PROTOTYPES ****************************/

/* ---------------------- SOCKET SUBROUTINES -----------------------*/
int
	sockets_open(void);
/* --------------------- WIRELESS SUBROUTINES ----------------------*/
int
	get_range_info(int		skfd,
		       char *		ifname,
		       iwrange *	range);

/* -------------------- FREQUENCY SUBROUTINES --------------------- */
void
	float2freq(double	in,
		   iwfreq *	out);


double
	freq2float(iwfreq *	in);
/* --------------------- ADDRESS SUBROUTINES ---------------------- */
int
	check_addr_type(int	skfd,
			char *	ifname);

char *pr_ether(unsigned char *ptr); 


int iw_getstats( char *ifname, struct iw_statistics *stats );


int freq2channel( float megaFreq, char *channel );

void getMin( int *minSignal, int *minNoise, int *minQual );

char *getmacString( unsigned char *mac);

int percent( int value, int min, int max );  

int getIps( int count, char **mac, char **ip );

char * getToken( char *line, int *i );


#endif
