/*
 * BSD specific module.
 *
 * For now, only talks to wi device.
 *
 * Lucent/orinoco cards have full range of signal/noise/quality.
 * Only prism2 card tested so far is D-link.  D-link
 *	signal ranges between 0..255.  no noise, therefore no quality.
 *
 * It would be nice if the wi driver would return the hw/firmware values.
 * hw type could be used to distinquish between lucent/prism2.
 * For now, we need a user hint for the prism cards.
 */
#include "cSigStat.h"
#include <stdio.h>

#include "wibsd_wifi.h"

#define FAILURE -1
#define SUCCESS 0

extern int prism2;

static int apflag = 0;

/**************************************************
 * Implementation of class cSigStat               *
 **************************************************/
cSigStat::cSigStat() 
{
}

cSigStat::~cSigStat() 
{
	if (apflag) {
		wi_wifi_scanap_stop();
	}
}

/* Initialize:
 * try and initialize named device.  
 * Sets internal name of interface in this module.
 * returns:
 *	FAILURE
 *	SUCCESS
 */
int cSigStat::Initialize( char *ifname, int scanap)
{
	int rc = 0;
	
	if (ifname == NULL)
		return FAILURE;

	/* set interface name
	*/
	wi_wifi_init(ifname);

	/* try and get the mode to see if we fail or
	 * not
	 */
	rc = wi_wifi_getmode();
	if (rc < 0)
		return FAILURE;

	apflag = scanap;

	if (apflag) {
		wi_wifi_scanap_start();
	}

	return SUCCESS;
}

/*
 * getmode: adhoc or managed.
 *
 * note mapping here of os/device specific into
 * wscan constants.
 */
int cSigStat::wifi_getmode() 
{
	int rc;
  
	rc = wi_wifi_getmode();
	if (rc < 0)
		return FAILURE;
	/* we make the gui think we are in ad hoc mode
	*/
	if (apflag)
		return(ADHOC_MODE);	
	if (rc == 2)
		return(ADHOC_MODE);	/* lucent ad hoc demo mapped to IBSS */
	if (rc == 1)
		return(ADHOC_MODE);	/* IBSS */
	return(MANAGED_MODE);
}

/*
 * getessid.
 * Managed mode only routine.  Also 
 * assumption here is that we can get only
 * one AP.
 */
char *cSigStat::wifi_getessid() 
{
	return(wi_wifi_getessid());
}

/*
 * getchannel.
 * managed mode only.
 */
char *cSigStat::wifi_getchannel()
{
	static char chanbuf[1024];
	int chanbase = 2412;

	int c = wi_wifi_getchan();
	chanbase = chanbase + ((c - 1) * 5);
  
	sprintf(chanbuf, "%d:_%d_Mhz", c, chanbase);
	return(chanbuf);
}

/*
 * getmac
 *
 * get mac id; i.e., managed mode mac address of ap.
 */
unsigned
char *cSigStat::wifi_getmac(unsigned char *addr) 
{
	memcpy(addr, (char *) (char *)wi_wifi_getmac(), 6);
	return(NULL);
}

/*
 * getrange.  let wscan know the range of signal strength, min/max.
 */
int cSigStat::wifi_getrange( int *maxLevel, int *maxNoise, int *maxQual,
	int *minLevel, int *minNoise, int *minQual)
{
	extern int prism2;

	if (prism2) {
		/* only level seems to exist */
		*maxLevel = 255;
		*minLevel = *minNoise = *minQual = 0;
		*maxNoise = 255; *maxQual = 255;
		return(SUCCESS);
	}
	(void) wi_wifi_getrange(maxLevel, maxNoise, maxQual);

	*minLevel = 47;
	*minNoise = 47;
	*minQual = 0;
	return(SUCCESS);
}


/*
 * managed mode only. assumption is that we cannot get
 * more than that.
 */
int cSigStat::wifi_getsignal( int *signal, int *noise, int *qual ) 
{
	int rc;

	rc = wi_wifi_getsignal(signal, noise, qual);
	if (rc < 0)
		return(FAILURE);
	return(SUCCESS);
	
}

#define APNEEDTOP	1
#define APNEEDBOTTOM    2

/*
 * gettable. ad hoc mode only.
 */
int cSigStat::wifi_gettable( int maxcount, sigtuple *st )
{
	int rc = 0;
	static int apstate = 0;

	if (apflag) {
		rc = wi_wifi_scanap(maxcount, st);
	}
	else {
		rc = wi_wifi_gettable(maxcount, st);
	}
	if (rc < 0)
		rc = 0;
	return(rc);
}

/*
 * since we are currently wi specific, and this function
 * is really being asked to guess the interface.  That is
 * all we do for now ... guess wi0.
 */
int cSigStat::wifi_getifs( int *count,  char **iflist, int max)
{ 
	*count = 1;
	iflist[0] = "wi0";
	return(SUCCESS);
}

void cSigStat::wifi_close()
{
	if (apflag) {
		wi_wifi_scanap_stop();
	}
}
