/*
 *
 * bsdwifi.c - based on wicontrol.c.  API routines for wifi
 *	&& lucent cards (if_wi.c, wi driver) on FreeBSD.
 *
 * Copyright (c) 1997, 1998, 1999, 2001
 *	Bill Paul <wpaul@ctr.columbia.edu>.  All rights reserved.
 *	Jim Binkley also hacked on this and transformed it into
 *	something else entirely, but Bill is still in there somewhere.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Bill Paul, or Jim Binkley
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY Bill Paul AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL Bill Paul OR THE VOICES IN HIS HEAD
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include <sys/types.h>
#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <net/if.h>
#include <net/if_var.h>
#include <net/ethernet.h>

#define WICACHE
#include <machine/if_wavelan_ieee.h>

#define APSCAN

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <err.h>

#include "sigtuple.h"

#if !defined(lint)
static const char copyright[] = "@(#) Copyright (c) 1997, 1998, 1999\
	Bill Paul. All rights reserved.";
#endif

#ifdef TESTMAIN
int debug = 1;
#else
int debug = 0;
#endif

static int  wi_getval		__P((char *, struct wi_req *));
static int  wi_setval		__P((char *, struct wi_req *));

static int 
wi_getval(iface, wreq)
char		*iface;
struct wi_req	*wreq;
{
	struct ifreq		ifr;
	int			s;

	bzero((char *)&ifr, sizeof(ifr));

	strlcpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
	ifr.ifr_data = (caddr_t)wreq;

	s = socket(AF_INET, SOCK_DGRAM, 0);

	if (s == -1)
		return(-1);

	if (ioctl(s, SIOCGWAVELAN, &ifr) == -1)
		return(-1);

	close(s);

	return(0);
}

static int 
wi_setval(iface, wreq)
char			*iface;
struct wi_req		*wreq;
{
	struct ifreq		ifr;
	int			s;

	bzero((char *)&ifr, sizeof(ifr));

	strlcpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
	ifr.ifr_data = (caddr_t)wreq;

	s = socket(AF_INET, SOCK_DGRAM, 0);

	if (s == -1)
		return(-1);

	if (ioctl(s, SIOCSWAVELAN, &ifr) == -1)
		return(-1);

	close(s);

	return(0);
}

void 
wi_fetchstr(wreq, sptr, maxsize)
struct wi_req	*wreq;
char *sptr;
int maxsize;
{
	int	i;
	char *ptr;

	if (wreq->wi_type == WI_RID_SERIALNO) {
		ptr = (char *)&wreq->wi_val;
		for (i = 0; i < (wreq->wi_len - 1) * 2; i++) {
			if (ptr[i] == '\0')
				ptr[i] = ' ';
		}
	} else {
		ptr = (char *)&wreq->wi_val[1];
		for (i = 0; i < wreq->wi_val[0]; i++) {
			if (ptr[i] == '\0')
				ptr[i] = ' ';
		}
	}

	ptr[i] = '\0';
	strncpy(sptr, ptr, maxsize);
	/*
	printf("[ %s ]\n", ptr);
	*/
	return;
}

void wi_gethex(wreq, mac, maxmac)
	struct wi_req		*wreq;
	char *mac;
	int maxmac;
{
	int			i;
	unsigned char		*c;

	c = (unsigned char *)&wreq->wi_val;

	/*
	printf("[ ");
	*/
	for (i = 0; i < (wreq->wi_len - 1) * 2; i++) {
		/*
		printf("%02x", c[i]);
		*/
		if (i == maxmac) 
			return;
		mac[i] = c[i];
		/*
		if (i < ((wreq->wi_len - 1) * 2) - 1)
			printf(":");
		*/
	}

	/*
	printf(" ]");
	*/
	return;
}

#define WI_STRING		0x01
#define WI_BOOL			0x02
#define WI_WORDS		0x03
#define WI_HEXBYTES		0x04
#define WI_KEYSTRUCT		0x05

struct wi_table {
	int			wi_code;
	int			wi_type;
	char			*wi_str;
};

#define ESSIDTYPE       3
#define MACTYPE		5
#define CHANTYPE	8
#define QUALTYPE	9
#define PORTTYPE	11
#define IBSSTYPE      	16
 
static struct wi_table wi_table[] = {
	/* 0 */
	{ WI_RID_SERIALNO, WI_STRING, "NIC serial number:\t\t\t" },
	/* 1 */
	{ WI_RID_NODENAME, WI_STRING, "Station name:\t\t\t\t" },
	/* 2 */
	{ WI_RID_OWN_SSID, WI_STRING, "SSID for IBSS creation:\t\t\t" },
	/* 3 */
	{ WI_RID_CURRENT_SSID, WI_STRING, "Current netname (SSID):\t\t\t" },
	/* 4 */
	{ WI_RID_DESIRED_SSID, WI_STRING, "Desired netname (SSID):\t\t\t" },
	/* 5 */
	{ WI_RID_CURRENT_BSSID, WI_HEXBYTES, "Current BSSID:\t\t\t\t" },
	/* 6 */
	{ WI_RID_CHANNEL_LIST, WI_WORDS, "Channel list:\t\t\t\t" },
	/* 7 */
	{ WI_RID_OWN_CHNL, WI_WORDS, "IBSS channel:\t\t\t\t" },
	/* 8 */
	{ WI_RID_CURRENT_CHAN, WI_WORDS, "Current channel:\t\t\t" },
	/* 9 */
	{ WI_RID_COMMS_QUALITY, WI_WORDS, "Comms quality/signal/noise:\t\t" },
	/* 10 */
	{ WI_RID_PROMISC, WI_BOOL, "Promiscuous mode:\t\t\t" },
	/* 11 */
	{ WI_RID_PORTTYPE, WI_WORDS, "Port type (1=BSS, 3=ad-hoc):\t\t"},
	/* 12 */
	{ WI_RID_MAC_NODE, WI_HEXBYTES, "MAC address:\t\t\t\t"},
	/* 13 */
	{ WI_RID_TX_RATE, WI_WORDS, "TX rate (selection):\t\t\t"},
	/* 14 */
	{ WI_RID_CUR_TX_RATE, WI_WORDS, "TX rate (actual speed):\t\t\t"},
	/* 15 */
	{ WI_RID_RTS_THRESH, WI_WORDS, "RTS/CTS handshake threshold:\t\t"},
	/* 16 */
	{ WI_RID_CREATE_IBSS, WI_BOOL, "Create IBSS:\t\t\t\t" },
	/* 17 */
	{ WI_RID_SYSTEM_SCALE, WI_WORDS, "Access point density:\t\t\t" },
	/* 18 */
	{ WI_RID_PM_ENABLED, WI_WORDS, "Power Mgmt (1=on, 0=off):\t\t" },
	/* 19 */
	{ WI_RID_MAX_SLEEP, WI_WORDS, "Max sleep time:\t\t\t\t" },
	{ 0, NULL }
};

static
char *
wi_getstring(iface, type, ptr, maxsize)
char *iface;
int type;
char *ptr;
int maxsize;
{
	struct wi_req		wreq;
	struct wi_table		*w;
	int rc;

	w = wi_table;
	bzero((char *)&wreq, sizeof(wreq));

	wreq.wi_len = WI_MAX_DATALEN;
	wreq.wi_type = w[type].wi_code;
	rc = wi_getval(iface, &wreq);
	if (rc < 0)
		return(NULL);
	wi_fetchstr(&wreq, ptr, maxsize);

	/*
	printf("%d\n", wreq->wi_len - 1);
	printf("%d ", wreq->wi_val[0]);
	return(wreq.wi_val[0]);
	*/
	return(ptr);
}

/*
 * so far:
 *	PORTTYPE and IBSSTYPE
 * returns:
 *	-1: error
 *	else int    
 */
static
int wi_getint(iface, type)
	char *iface;
	int  type;
{
	struct wi_req		wreq;
	struct wi_table		*w;
	int rc;

	w = wi_table;
	bzero((char *)&wreq, sizeof(wreq));

	wreq.wi_len = WI_MAX_DATALEN;
	wreq.wi_type = w[type].wi_code;
	rc = wi_getval(iface, &wreq);
	if (rc < 0)
		return(rc);

	/*
	printf("%d\n", wreq->wi_len - 1);
	printf("%d ", wreq->wi_val[0]);
	*/
	return(wreq.wi_val[0]);
}

/*
 * so far:
 *	MACTYPE
 * returns:
 *	-1: error
 *	else short char string
 */
static
int
wi_getmac(iface, type, mac, maxmac)
	char *iface;
	int  type;
	char *mac;
	int maxmac;
{
	struct wi_req		wreq;
	struct wi_table		*w;
	int rc;

	w = wi_table;
	bzero((char *)&wreq, sizeof(wreq));

	wreq.wi_len = WI_MAX_DATALEN;
	wreq.wi_type = w[type].wi_code;
	rc = wi_getval(iface, &wreq);
	if (rc < 0)
		return(rc);

	wi_gethex(&wreq, mac, maxmac);

	return(0);
	/*
	printf("%d\n", wreq->wi_len - 1);
	printf("%d ", wreq->wi_val[0]);
	*/

}

static
int
wi_getsignal(iface, type, quality, signal, noise)
	char *iface;
	int  type;
	int *quality;
	int *signal;
	int *noise;
{
	struct wi_req		wreq;
	struct wi_table		*w;
	int rc;

	w = wi_table;
	bzero((char *)&wreq, sizeof(wreq));

	wreq.wi_len = WI_MAX_DATALEN;
	wreq.wi_type = w[type].wi_code;
	rc = wi_getval(iface, &wreq);
	if (rc < 0)
		return(rc);

	*quality = wreq.wi_val[0];
	*signal = wreq.wi_val[1];
	*noise = wreq.wi_val[2];

	return(0);
}



/* --------------------- wifi api routines ---------------------- */

/* wifi functions
*/

static
char wifi_if[128];	/* name of interface in use */

/*
 * initialize the module:
 *
 * inputs:
 *	if must be the string name of the interface, else wifi_init
 *	will provide a hardwired guess.
 */
void
wi_wifi_init(char *iface)
{
	if (iface == NULL) {
		strncpy(wifi_if, "wi0", 128);
	}
	else {
		strncpy(wifi_if, iface, 128);
	}
}

/*
 * wifi_getmode
 *
 * returns:
 *     -1:  fatal error
 *	0 - managed mode
 *	1 - IBSS mode
 *	2 - lucent ad hoc demo mode.
 */
int 
wi_wifi_getmode()
{
	int ibss;	/* ibss flag, 0 or 1 (off/on) */
	int wimode = 0;

	/* returns 1 or 3 for porttype
	*/
	wimode = wi_getint(wifi_if, PORTTYPE);
#ifdef TESTMAIN
	if (debug) {
		printf("wifi_getmode, port type %d\n", wimode);
	}
#endif
	if (wimode < 0)
		return(-1);	

	/* weed out lucent ad hoc demo mode
	*/
	if (wimode == 3) {
#ifdef TESTMAIN
		if (debug) {
			printf("wifi_getmode, lucent ad hoc demo, returns 2\n");
		}
#endif
		return(2);
	}
	/* figure out if ibss or not
	*/ 
	ibss = wi_getint(wifi_if, IBSSTYPE);
	if (debug) {
		printf("wifi_getmode, ibss type %d\n", ibss);
	}
	if (ibss) {
		return(1);
	}
	return(0);
}

/*
 * return 
 *	0: on error
 * 	else: channel in range 1..whatever
 * 
*/
int
wi_wifi_getchan()
{
	int wimode;
	int chan;

	wimode = wi_getint(wifi_if, PORTTYPE);
	if (wimode < 0)  /* check for failure */
		return(0);
	chan = wi_getint(wifi_if, CHANTYPE);
#ifdef TESTMAIN
	if (debug) {
		printf("wifi_getchan mode %d, chan %d\n", wimode, chan);
	}
#endif
	return(chan);
}

/*
 * get the essid string and return it
 * Note: we do this in static storage, so
 * the caller need not worry about volatility.
 * returns:
 *	NULL - any error 
 *	string - if things work out
 */
char *
wi_wifi_getessid()
{
	static char essid[128];
	int wimode;

	bzero(essid, 128);
	wimode = wi_getint(wifi_if, PORTTYPE);
	if (wimode < 0)  /* check for failure */
		return(NULL);
	wi_getstring(wifi_if, ESSIDTYPE, essid, 128);
#ifdef TESTMAIN
	if (debug) {
		printf("essid: [%s]\n", essid);
	}
#endif
	return(essid);
}

char *
wi_wifi_getmac()
{
	static char mac[128];
	int wimode;

	bzero(mac, 128);
	wimode = wi_getint(wifi_if, PORTTYPE);
	if (wimode < 0)  /* check for failure */
		return(NULL);

	wi_getmac(wifi_if, MACTYPE, mac, 128);
#ifdef TESTMAIN
	if (debug) {
		printf("wifi_getmac: [%02x:%02x:%02x:%02x:%02x:%02x]\n",
			mac[0], mac[1], mac[2], 
			mac[3], mac[4], mac[5]);
	}
#endif
	return(mac);
}

void
wi_wifi_getrange(signal, noise, quality)
int *signal;
int *noise;
int *quality;
{
	*signal = *noise = 138;
	*quality = 138 - 47;
}

/*
 * works only in managed mode:
 * return signal/noise/quality for single AP.
 *
 * if called in non-managed mode, simply returns 0's.
 * if no device, returns -1
 */
int
wi_wifi_getsignal(signal, noise, quality)
int *signal;
int *noise;
int *quality;
{
	int wimode;

	wimode = wi_getint(wifi_if, PORTTYPE);
	/* TBD. fix to only work with managed mode
	*/
	if (wimode < 0)  { /* check for failure */
		*signal = *noise = *quality = 0;
		return(-1);
	}

	/* note: params here in order of hardware readout, not wifi API 
	*/
	wi_getsignal(wifi_if, QUALTYPE, quality, signal, noise);
#ifdef TESTMAIN
	if (debug) {
		printf("signal noise quality, %d %d %d\n",
			*signal, *noise, *quality);
	}
#endif
	/* TBD. scale the result to 0..100%
	*/
	return(0);
}


/*
 * this routine works with the wi cache, and is assumed to
 * only be used in ad hoc mode.
 *
 * returns: 
 *   rc: direct integer return
 *	0, no if or no table
 *	>1: # of entries in table, rc <= maxcount by definition
 *	st: filled in with table entries for > 0
 *	
 */
int
wi_wifi_gettable(maxcount, st)
	int 	maxcount;	/* input only, max table values */
	sigtuple *st;
{
	struct wi_req		wreq;
	int 			*wi_sigitems;
	struct wi_sigcache 	*sc;
	char *			pt;
	int 			i;
	int 			rc;	/* # of returned items */
	int wimode;
	char *iface;

	/* it is not possible to get more than MAXWICACHE entries
	 * out of the driver
	*/
	if (maxcount > MAXWICACHE)
		maxcount = MAXWICACHE;

	wimode = wi_getint(wifi_if, PORTTYPE);
	/* TBD. fix to only work with adhoc mode
	*/ 
	if (wimode < 0)  { /* check for failure */
		return(0);
	}
	iface = wifi_if;

	bzero((char *)&wreq, sizeof(wreq));
	wreq.wi_len = WI_MAX_DATALEN;
	wreq.wi_type = WI_RID_READ_CACHE;

	wi_getval(iface, &wreq);

	wi_sigitems = (int *) &wreq.wi_val; 
	pt = ((char *) &wreq.wi_val);
	pt += sizeof(int);
	sc = (struct wi_sigcache *) pt;

	rc = *wi_sigitems;
	if (rc > maxcount)
		rc = maxcount;

	/* transpose table entries
	*/
	for ( i = 0; i < rc; i++) {
		st->signal = sc->signal + 149;
		st->noise = sc->noise + 149;
		st->quality = sc->quality;
		bcopy((char *)sc->macsrc, (char *)st->mac, 6);
		st->ip = sc->ipsrc;
		sc++; st++;
		bzero(st->essid, 33);
	}

	return(rc);
}

#ifdef APSCAN
/*
 * AP scan in current driver is interrupt driven.
 * You can start/stop/or poll the driver.  The
 * driver does all the work.  In theory, this
 * should make our life easy here.  We just start
 * scanning and read out the table info as long
 * as we want.  scanning occurs in the driver at a
 * 1 second rate.
 * 
 * start:
 *	-1: failure (no if most likely, but you cannot do start
 *		twice without a stop inbetween) 
 *	 0: worked
 */ 
int 
wi_wifi_scanap_start() 
{
        struct wi_req           wreq;
	int wimode;
	char *iface;
	int rc;

	/* port type is basically a sanity check at this point.
	 * Ironically ap scans do NOT care about the mode.  They
	 * just work.
	 */
	iface = wifi_if;
	wimode = wi_getint(wifi_if, PORTTYPE);
	if (wimode < 0)  { /* check for failure */
		return(-1);
	}

        bzero((char *)&wreq, sizeof(wreq));

        wreq.wi_type = WI_RID_SCAN_APS;

        /* write the request
        */
        rc = wi_setval(iface, &wreq);
	return(rc);
}

int 
wi_wifi_scanap_stop() 
{
        struct wi_req           wreq;
	int wimode;
	char *iface;
	int rc;

	/* port type is basically a sanity check at this point.
	 * Ironically ap scans do NOT care about the mode.  They
	 * just work.
	 */
	iface = wifi_if;
	wimode = wi_getint(wifi_if, PORTTYPE);
	if (wimode < 0)  { /* check for failure */
		return(-1);
	}
        bzero((char *)&wreq, sizeof(wreq));
        wreq.wi_type = WI_RID_STOP_SCAN_APS;

        /* write the request
        */
        rc = wi_setval(iface, &wreq);
	return(rc);
}

/*
 * 
 * returns:
 *	 direct (rc): count of naps
 *		note: 0 is of course possible.
 *	-1: not ready.
 *	 0: data returned in sigtuple/table st.
 */

int
wi_wifi_scanap(maxcount, st)
	int maxcount;
	sigtuple *st;
{
        int s;
	int i;
        struct wi_req           wreq;
        struct ifreq            ifr;
        int naps;
        struct wi_apinfo *w;
	char *iface = wifi_if;

        bzero((char *)&wreq, sizeof(wreq));
        wreq.wi_len = WI_MAX_DATALEN;
        wreq.wi_type = WI_RID_READ_APS;

        strlcpy(ifr.ifr_name, iface, sizeof(ifr.ifr_name));
        ifr.ifr_data = (caddr_t)&wreq;

        s = socket(AF_INET, SOCK_DGRAM, 0);
        if (s == -1)
                return(-1);

        if (ioctl(s, SIOCGWAVELAN, &ifr) == -1) {
		close(s);
		return(-1);
	}
	close(s);

	naps = (int) wreq.wi_val[0]; 
	if (naps > maxcount)
		naps = maxcount;
	w =  (struct wi_apinfo *) (((char *)&wreq.wi_val) + sizeof(int));
	for ( i = 0; i < naps; i++, w++) {
		/* signal/noise/quality */
                st->signal = w->signal + 149;  /* convert FROM negative */
                st->noise = w->noise + 149;
                st->quality = w->quality;

		/* ap mac address */
                bcopy((char *)w->bssid, (char *)st->mac, 6);
		/* there is no ip value */
		/* name of network (essid) */
		bzero(st->essid,33);
		if (w->namelen > 32)
			w->namelen = 32;   
		bcopy((char *)w->name, st->essid, w->namelen); 
		st->channel = w->channel;
		st->wep = 0;
		if (w->capinfo & 0x010) 
			st->wep = 1;
                st++;
	}
	return(naps);
}
#endif

#ifdef TESTMAIN

void
dump_gettable(rc, count, sc)
int rc;
int count;
sigtuple *sc;
{
	int i;

	printf("rc (real values) %d, count (max values) %d\n", rc, count);
	for (i = 0; i < rc; i++) {
		printf("[%d/%d]:", i+1, rc);
		printf(" %02x:%02x:%02x:%02x:%02x:%02x,",
		  		    	sc->mac[0]&0xff,
		  		    	sc->mac[1]&0xff,
		   		    	sc->mac[2]&0xff,
		   			sc->mac[3]&0xff,
		   			sc->mac[4]&0xff,
		   			sc->mac[5]&0xff);
		if (sc->essid[0] == 0)
        		printf(" %d.%d.%d.%d,",((sc->ip >> 0) & 0xff),
				        ((sc->ip >> 8) & 0xff),
				        ((sc->ip >> 16) & 0xff),
				        ((sc->ip >> 24) & 0xff));
		printf(" sig: %d, noise: %d, qual: %d\n",
		   			sc->signal,
		   			sc->noise,
		   			sc->quality);
		if (sc->essid[0]) {
			printf(" essid: %s, channel %d\n", sc->essid, 
				sc->channel);
			if (sc->wep) {
				printf("wep!\n");
			}
		}
		sc++;
	}
}


static void usage(p)
	char			*p;
{

	/* DONE. get mode */
	fprintf(stderr, "\t%s -i iface -p port type\n", p);
	/* DONE. get essid */
	fprintf(stderr, "\t%s -i iface -n network name\n", p);
	/* DONE. get channel */
	fprintf(stderr, "\t%s -i iface -f frequency\n", p);

	/* get mac address */
	fprintf(stderr, "\t%s -i iface -m mac address\n", p);
	/* get signal tuple in managed mode only */
	fprintf(stderr, "\t%s -i iface -s managed AP signal strength \n", p);
	/* get signal table in any mode */
	fprintf(stderr, "\t%s -i iface -C print signal cache\n", p);
#ifdef APSCAN
	fprintf(stderr, "\t%s -i iface -A ap scan\n", p);
#endif

	exit(1);
}

int main(argc, argv)
	int			argc;
	char			*argv[];
{
	int			ch;
	char			*iface = NULL;
	char			*p = argv[0];
	char			*key = NULL;
	int			modifier = 0;

	/* Get the interface name */
	opterr = 0;
	ch = getopt(argc, argv, "i:");
	if (ch == 'i') {
		iface = optarg;
	} else {
		iface = "wi0";
		optreset = 1;
		optind = 1;
	}
	opterr = 1;
	wi_wifi_init(iface);
		
	while((ch = getopt(argc, argv, "ApfnmsCi:")) != -1) {
		switch(ch) {
#ifdef APSCAN
		case 'A':
			{
				int rc;
				int count;
				sigtuple st[6];
				rc = wi_wifi_scanapTOP();
				if (rc < 0) {
					perror("scanapTOP");
					exit(1);
				}
				count = 6;
				while(1) {
					rc = wi_wifi_scanapBOT(count, &st);
					if (rc < 0) {
						;
					}
					else {
						dump_gettable(rc, count, &st);
						break;
					}
	 			}
			}
			exit(0);
			break;
#endif
		case 'p':
			wi_wifi_getmode();
			exit(0);
			break;
		case 'f':
			wi_wifi_getchan();
			exit(0);
			break;
		case 'n':
			wi_wifi_getessid();
			exit(0);
			break;
		/* mac address */
		case 'm':
			wi_wifi_getmac();
			exit(0);
			break;
		/* single managed mode only signal strength
		*/
		case 's':
			{
				int s;  int q;   int n;
				wi_wifi_getsignal(&s, &n, &q);
			}
			exit(0);
			break;
		case 'C':
			{
				int count;
				int rc;
				sigtuple st[6];

				count = 6;
				rc = wi_wifi_gettable(count, &st);
				dump_gettable(rc, count, &st);
			}
			exit(0);
			break;
#ifdef OLD
		case 'Z':
#ifdef WICACHE
			wi_zerocache(iface);
			exit(0);
#else
			printf("WICACHE not available\n");
#endif
			break;
		case 'C':
#ifdef WICACHE
			wi_readcache(iface);
#else
			printf("WICACHE not available\n");
#endif
			exit(0);
			break;
		case 'c':
			wi_setword(iface, WI_RID_CREATE_IBSS, atoi(optarg));
			exit(0);
			break;
		case 'd':
			wi_setword(iface, WI_RID_MAX_DATALEN, atoi(optarg));
			exit(0);
			break;
		case 'e':
			wi_setword(iface, WI_RID_ENCRYPTION, atoi(optarg));
			exit(0);
			break;
		case 'f':
			wi_setword(iface, WI_RID_OWN_CHNL, atoi(optarg));
			exit(0);
			break;
		case 'k':
			key = optarg;
			break;
		case 'r':
			wi_setword(iface, WI_RID_RTS_THRESH, atoi(optarg));
			exit(0);
			break;
		case 't':
			wi_setword(iface, WI_RID_TX_RATE, atoi(optarg));
			exit(0);
			break;
		case 'n':
			wi_setstr(iface, WI_RID_DESIRED_SSID, optarg);
			exit(0);
			break;
		case 's':
			wi_setstr(iface, WI_RID_NODENAME, optarg);
			exit(0);
			break;
		case 'm':
			wi_sethex(iface, WI_RID_MAC_NODE, optarg);
			exit(0);
			break;
		case 'q':
			wi_setstr(iface, WI_RID_OWN_SSID, optarg);
			exit(0);
			break;
		case 'S':
			wi_setword(iface, WI_RID_MAX_SLEEP, atoi(optarg));
			exit(0);
			break;
		case 'T':
			wi_setword(iface,
			    WI_RID_TX_CRYPT_KEY, atoi(optarg) - 1);
			exit(0);
			break;
		case 'P':
			wi_setword(iface, WI_RID_PM_ENABLED, atoi(optarg));
			exit(0);
			break;
		case 'v':
			modifier = atoi(optarg);
			modifier--;
			break;
		case 'h':
#endif
		default:
			usage(p);
			break;
		}
	}

	usage(p);

	exit(0);
}
#endif
